@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2001-2011, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following two licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -313,7 +313,15 @@ int xs__schema::read(const char *cwd, const char *loc)
   if (vflag)
     fprintf(stderr, "\nOpening schema '%s' from '%s'\n", loc?loc:"", cwd?cwd:"");
   if (loc)
-  {
+  { if (soap->recvfd > 2)
+    { soap_end_recv(soap);
+      close(soap->recvfd);
+      soap->recvfd = -1;
+    }
+    else if (soap_valid_socket(soap->socket))
+    { soap_end_recv(soap);
+      soap_closesock(soap);
+    }
 #ifdef WITH_OPENSSL
     if (!strncmp(loc, "http://", 7) || !strncmp(loc, "https://", 8))
 #else
@@ -469,12 +477,19 @@ int xs__include::preprocess(xs__schema &schema)
 { if (!schemaRef && schemaLocation)
   { // only read from include locations not read already, uses static std::map
     static map<const char*, xs__schema*, ltstr> included;
-    map<const char*, xs__schema*, ltstr>::iterator i = included.find(schemaLocation);
+    map<const char*, xs__schema*, ltstr>::iterator i = included.end();
+    if (schema.targetNamespace)
+      for (i = included.begin(); i != included.end(); ++i)
+        if ((*i).second->targetNamespace
+         && !strcmp(schemaLocation, (*i).first)
+         && !strcmp(schema.targetNamespace, (*i).second->targetNamespace))
+          break;
     if (i == included.end())
     { if (vflag)
         cerr << "Preprocessing schema include '" << (schemaLocation?schemaLocation:"") << "' into schema '" << (schema.targetNamespace?schema.targetNamespace:"") << "'" << endl;
       included[schemaLocation] = schemaRef = new xs__schema(schema.soap);
       schemaRef->read(schema.sourceLocation(), schemaLocation);
+      schemaRef->targetNamespace = schema.targetNamespace;
     }
     else
     { if (vflag)
@@ -756,12 +771,13 @@ int xs__element::traverse(xs__schema &schema)
   elementRef = NULL;
   if (token)
   { for (vector<xs__element>::iterator i = schema.element.begin(); i != schema.element.end(); ++i)
-      if (!strcmp((*i).name, token))
+    { if ((*i).name && !strcmp((*i).name, token))
       { elementRef = &(*i);
         if (vflag)
           cerr << "    Found element '" << (name?name:"") << "' ref '" << (token?token:"") << "'" << endl;
         break;
       }
+    }
   }
   if (!elementRef)
   { for (vector<xs__import>::const_iterator i = schema.import.begin(); i != schema.import.end(); ++i)
@@ -770,7 +786,7 @@ int xs__element::traverse(xs__schema &schema)
       { token = qname_token(ref, s->targetNamespace);
         if (token)
         { for (vector<xs__element>::iterator j = s->element.begin(); j != s->element.end(); ++j)
-          { if (!strcmp((*j).name, token))
+          { if ((*j).name && !strcmp((*j).name, token))
             { elementRef = &(*j);
               if (vflag)
                 cerr << "    Found element '" << (name?name:"") << "' ref '" << (token?token:"") << "'" << endl;
@@ -792,7 +808,7 @@ int xs__element::traverse(xs__schema &schema)
     simpleTypeRef = NULL;
     if (token)
     { for (vector<xs__simpleType>::iterator i = schema.simpleType.begin(); i != schema.simpleType.end(); ++i)
-        if (!strcmp((*i).name, token))
+        if ((*i).name && !strcmp((*i).name, token))
         { simpleTypeRef = &(*i);
           if (vflag)
             cerr << "    Found element '" << (name?name:"") << "' simpleType '" << (token?token:"") << "'" << endl;
@@ -806,7 +822,7 @@ int xs__element::traverse(xs__schema &schema)
         { token = qname_token(type, s->targetNamespace);
           if (token)
           { for (vector<xs__simpleType>::iterator j = s->simpleType.begin(); j != s->simpleType.end(); ++j)
-            { if (!strcmp((*j).name, token))
+            { if ((*j).name && !strcmp((*j).name, token))
               { simpleTypeRef = &(*j);
                 if (vflag)
                   cerr << "    Found element '" << (name?name:"") << "' simpleType '" << (token?token:"") << "'" << endl;
@@ -829,7 +845,7 @@ int xs__element::traverse(xs__schema &schema)
     complexTypeRef = NULL;
     if (token)
     { for (vector<xs__complexType>::iterator i = schema.complexType.begin(); i != schema.complexType.end(); ++i)
-        if (!strcmp((*i).name, token))
+        if ((*i).name && !strcmp((*i).name, token))
         { complexTypeRef = &(*i);
           if (vflag)
             cerr << "    Found element '" << (name?name:"") << "' complexType '" << (token?token:"") << "'" << endl;
@@ -843,7 +859,7 @@ int xs__element::traverse(xs__schema &schema)
         { token = qname_token(type, s->targetNamespace);
           if (token)
           { for (vector<xs__complexType>::iterator j = s->complexType.begin(); j != s->complexType.end(); ++j)
-            { if (!strcmp((*j).name, token))
+            { if ((*j).name && !strcmp((*j).name, token))
               { complexTypeRef = &(*j);
                 if (vflag)
                   cerr << "    Found element '" << (name?name:"") << "' complexType '" << (token?token:"") << "'" << endl;
@@ -1149,7 +1165,7 @@ int xs__extension::traverse(xs__schema &schema)
   complexTypeRef = NULL;
   if (token)
   { for (vector<xs__complexType>::iterator i = schema.complexType.begin(); i != schema.complexType.end(); ++i)
-      if (!strcmp((*i).name, token))
+      if ((*i).name && !strcmp((*i).name, token))
       { complexTypeRef = &(*i);
         if (vflag)
           cerr << "    Found extension base type '" << (token?token:"") << "'" << endl;
@@ -1163,7 +1179,7 @@ int xs__extension::traverse(xs__schema &schema)
       { token = qname_token(base, s->targetNamespace);
         if (token)
         { for (vector<xs__complexType>::iterator j = s->complexType.begin(); j != s->complexType.end(); ++j)
-          { if (!strcmp((*j).name, token))
+          { if ((*j).name && !strcmp((*j).name, token))
             { complexTypeRef = &(*j);
               if (vflag)
                 cerr << "    Found extension base type '" << (token?token:"") << "'" << endl;
@@ -1213,6 +1229,8 @@ xs__restriction::xs__restriction()
 int xs__restriction::traverse(xs__schema &schema)
 { if (vflag)
     cerr << "   Analyzing schema restriction '" << (base?base:"") << "'" << endl;
+  if (attributeGroup)
+    attributeGroup->traverse(schema);
   if (group)
     group->traverse(schema);
   else if (all)
@@ -1264,7 +1282,7 @@ int xs__restriction::traverse(xs__schema &schema)
   complexTypeRef = NULL;
   if (token)
   { for (vector<xs__complexType>::iterator i = schema.complexType.begin(); i != schema.complexType.end(); ++i)
-      if (!strcmp((*i).name, token))
+      if ((*i).name && !strcmp((*i).name, token))
       { complexTypeRef = &(*i);
         if (vflag)
           cerr << "    Found restriction base type '" << (token?token:"") << "'" << endl;
@@ -1278,7 +1296,7 @@ int xs__restriction::traverse(xs__schema &schema)
       { token = qname_token(base, s->targetNamespace);
         if (token)
         { for (vector<xs__complexType>::iterator j = s->complexType.begin(); j != s->complexType.end(); ++j)
-          { if (!strcmp((*j).name, token))
+          { if ((*j).name && !strcmp((*j).name, token))
             { complexTypeRef = &(*j);
               if (vflag)
                 cerr << "    Found restriction base type '" << (token?token:"") << "'" << endl;
