@@ -1,10 +1,10 @@
 /*
-	stdsoap2.h 2.8.7
+	stdsoap2.h 2.8.12
 
 	gSOAP runtime engine
 
 gSOAP XML Web services tools
-Copyright (C) 2000-2011, Robert van Engelen, Genivia Inc., All Rights Reserved.
+Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc., All Rights Reserved.
 This part of the software is released under ONE of the following licenses:
 GPL, or the gSOAP public license, or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -24,7 +24,7 @@ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the License.
 
 The Initial Developer of the Original Code is Robert A. van Engelen.
-Copyright (C) 2000-2011, Robert van Engelen, Genivia Inc., All Rights Reserved.
+Copyright (C) 2000-2012, Robert van Engelen, Genivia Inc., All Rights Reserved.
 --------------------------------------------------------------------------------
 GPL license.
 
@@ -51,7 +51,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 --------------------------------------------------------------------------------
 */
 
-#define GSOAP_H_VERSION 20807
+#define GSOAP_VERSION 20812
 
 #ifdef WITH_SOAPDEFS_H
 # include "soapdefs.h"		/* include user-defined stuff */
@@ -131,6 +131,13 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 # endif
 #endif
 
+/* for legacy purposes we use WIN32 macro even though 64 bit is supported */
+#ifdef _WIN64
+# ifndef WIN32
+#  define WIN32
+# endif
+#endif
+
 #ifdef _WIN32_WCE
 # ifndef UNDER_CE
 #  define UNDER_CE _WIN32_WCE
@@ -217,6 +224,10 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #  define HAVE_SPRINTF_L
 #  define HAVE_STRTOL
 #  define HAVE_STRTOUL
+#  if _MSC_VER >= 1300
+#   define HAVE_STRTOLL		// use _strtoi64
+#   define HAVE_STRTOULL	// use _strtoui64
+#  endif
 #  define HAVE_SYS_TIMEB_H
 #  define HAVE_FTIME
 #  define HAVE_GMTIME
@@ -297,7 +308,6 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #  define HAVE_WCTOMB
 #  define HAVE_MBTOWC
 #  define HAVE_ISNAN
-#  define HAVE_ISINF
 # elif defined(FREEBSD) || defined(__FreeBSD__) || defined(OPENBSD)
 #  define HAVE_POLL
 #  define HAVE_SNPRINTF
@@ -713,7 +723,13 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #  include <io.h>
 #  include <fcntl.h>
 # endif
-# include <winsock2.h> /* Visual Studio 2005 users: you must install the Platform SDK (R2) */
+// When you get macro redefinition errors when compiling the code below:
+// try arrange your include list that <windows.h> is included after "stdsoap2.h"
+// or define _WINSOCKAPI_ first:
+// #define _WINSOCKAPI_    // stops windows.h including winsock.h
+// #include <windows.h>
+// #include "stdsoap2.h"
+# include <winsock2.h> /* Visual Studio 2005 users: install Platform SDK (R2) */
 # include <ws2tcpip.h>
 // # define _WSPIAPI_COUNTOF /* DEV NOTE: enble to fix problems with VC6 */
 // # include <wspiapi.h>
@@ -814,9 +830,9 @@ extern "C" {
 # endif
 #elif defined(SOCKLEN_T)
 # define SOAP_SOCKLEN_T SOCKLEN_T
-#elif defined(__socklen_t_defined) || defined(_SOCKLEN_T) || defined(CYGWIN) || defined(FREEBSD) || defined(__FreeBSD__) || defined(OPENBSD) || defined(__QNX__) || defined(QNX) || defined(OS390) || defined(HP_UX)
+#elif defined(__socklen_t_defined) || defined(_SOCKLEN_T) || defined(CYGWIN) || defined(FREEBSD) || defined(__FreeBSD__) || defined(OPENBSD) || defined(__QNX__) || defined(QNX) || defined(OS390)
 # define SOAP_SOCKLEN_T socklen_t
-#elif defined(IRIX) || defined(WIN32) || defined(__APPLE__) || defined(SUN_OS) || defined(OPENSERVER) || defined(TRU64) || defined(VXWORKS)
+#elif defined(IRIX) || defined(WIN32) || defined(__APPLE__) || defined(SUN_OS) || defined(OPENSERVER) || defined(TRU64) || defined(VXWORKS) || defined(HP_UX)
 # define SOAP_SOCKLEN_T int
 #else
 # define SOAP_SOCKLEN_T size_t
@@ -1162,7 +1178,17 @@ extern const char soap_base64o[], soap_base64i[];
 #define soap_isninfd(n) ((n) < 0 && soap_isinf(n))
 #define soap_isninff(n) ((n) < 0 && soap_isinf(n))
 
-/* gSOAP error codes */
+#ifdef HAVE_SNPRINTF
+# ifdef WIN32
+#  define soap_snprintf _snprintf
+# else
+#  define soap_snprintf snprintf
+# endif
+#endif
+
+/* gSOAP status/error codes */
+
+typedef soap_int32 soap_status;
 
 #define SOAP_EOF			EOF
 #define SOAP_ERR			EOF
@@ -1230,7 +1256,7 @@ extern const char soap_base64o[], soap_base64i[];
 /* Exceptional gSOAP HTTP server response status codes >= 1000 */
 
 #define SOAP_STOP		1000	/* No HTTP response */
-#define SOAP_FORM		1001	/* Form request/response */
+#define SOAP_FORM		1001	/* Request (form) data is present, no HTTP response */
 #define SOAP_HTML		1002	/* Custom HTML response */
 #define SOAP_FILE		1003	/* Custom file-based response */
 
@@ -1239,7 +1265,9 @@ extern const char soap_base64o[], soap_base64i[];
 #define SOAP_POST		2000	/* POST request */
 #define SOAP_POST_FILE		2001	/* Custom file-based POST request */
 #define SOAP_GET		2002	/* GET request */
-#define SOAP_CONNECT		2003	/* CONNECT request */
+#define SOAP_PUT		2003	/* PUT request */
+#define SOAP_DEL		2004	/* DELETE request */
+#define SOAP_CONNECT		2005	/* CONNECT request */
 
 /* gSOAP DIME */
 
@@ -1287,7 +1315,7 @@ typedef soap_int32 soap_mode;
 #define SOAP_XML_CANONICAL	0x00010000	/* out: excC14N canonical XML */
 #define SOAP_XML_TREE		0x00020000	/* out: XML tree (no id/ref) */
 #define SOAP_XML_NIL		0x00040000	/* out: NULLs as xsi:nil */
-#define SOAP_XML_NOTYPE		0x00080000	/* out: NULLs as xsi:nil */
+#define SOAP_XML_NOTYPE		0x00080000	/* out: do not add xsi:type */
 
 #define SOAP_DOM_TREE		0x00100000      /* see DOM manual */
 #define SOAP_DOM_NODE		0x00200000
@@ -1352,6 +1380,7 @@ typedef soap_int32 soap_mode;
 
 #define SOAP_SEC_BEGIN		1
 #define SOAP_SEC_SIGN		2
+#define SOAP_SEC_DECRYPT	3
 
 /* DEBUG macros */
 
@@ -1383,11 +1412,27 @@ typedef soap_int32 soap_mode;
 # define SOAP_FREE(soap, ptr) free(ptr)
 #endif
 
-#ifndef SOAP_NEW				/* use C++ new operator */
-# if (defined(__GNUC__) && (__GNUC__ <= 2) && !defined(__BORLANDC__)) || defined(__clang__) || defined(_AIX)
-#  define SOAP_NEW(type) new type		/* old form w/o parenthesis */
-# else
-#  define SOAP_NEW(type) new (type)		/* prefer with parenthesis */
+#if !defined(WITH_LEAN) && !defined(WITH_COMPAT)
+# define SOAP_NOTHROW (std::nothrow)
+#else
+# define SOAP_NOTHROW
+#endif
+
+#if (defined(__GNUC__) && (__GNUC__ <= 2) && !defined(__BORLANDC__)) || defined(__clang__) || defined(_AIX)
+/* old form w/o parenthesis */
+# ifndef SOAP_NEW
+#  define SOAP_NEW(type) new SOAP_NOTHROW type
+# endif
+# ifndef SOAP_NEW_ARRAY
+#  define SOAP_NEW_ARRAY(type, n) new SOAP_NOTHROW type[n]
+# endif
+#else
+/* new form with parenthesis */
+# ifndef SOAP_NEW
+#  define SOAP_NEW(type) new SOAP_NOTHROW (type)
+# endif
+# ifndef SOAP_NEW_ARRAY
+#  define SOAP_NEW_ARRAY(type, n) new SOAP_NOTHROW (type[n])
 # endif
 #endif
 
@@ -1396,7 +1441,7 @@ typedef soap_int32 soap_mode;
 #endif
 
 #ifndef SOAP_NEW_COPY			/* use C++ new operator for ::copy() */
-# define SOAP_NEW_COPY(clas) new clas	/* prefer w/o parenthesis */
+# define SOAP_NEW_COPY(clas) new SOAP_NOTHROW clas
 #endif
 
 #ifndef SOAP_DELETE			/* use C++ delete operator */
@@ -1792,7 +1837,7 @@ extern "C" {
 
 struct SOAP_STD_API soap
 { short state;			/* 0 = uninitialized, 1 = initialized, 2 = copy of another soap struct */
-  short version;		/* 1 = SOAP1.1 and 2 = SOAP1.2 (set automatically from namespace URI in nsmap table) */
+  short version;		/* 1 = SOAP1.1 and 2 = SOAP1.2 (set automatically from namespace URI in nsmap table), 0 indicates non-SOAP content */
   soap_mode mode;
   soap_mode imode;
   soap_mode omode;
@@ -1801,7 +1846,7 @@ struct SOAP_STD_API soap
   const char *dime_id_format;	/* user-definable format string for integer DIME id (<SOAP_TAGLEN chars) */
   const char *http_version;	/* HTTP version used "1.0" or "1.1" */
   const char *http_content;	/* optional custom response content type (with SOAP_FILE) */
-  const char *encodingStyle;	/* default = NULL which means that SOAP encoding is used */
+  const char *encodingStyle;	/* default = "" which means that SOAP encoding is used */
   const char *actor;		/* SOAP-ENV:actor or role attribute value */
   const char *lang;		/* xml:lang attribute value of SOAP-ENV:Text */
   int recv_timeout;		/* when > 0, gives socket recv timeout in seconds, < 0 in usec */
@@ -1819,10 +1864,12 @@ struct SOAP_STD_API soap
   struct soap_blist *blist;	/* block allocation stack */
   struct soap_clist *clist;	/* class instance allocation list */
   void *alist;			/* memory allocation (malloc) list */
+#if !defined(WITH_LEAN) || !defined(WITH_NOIDREF)
   struct soap_ilist *iht[SOAP_IDHASH];
   struct soap_plist *pht[SOAP_PTRHASH];
   struct soap_pblk *pblk;	/* plist block allocation */
   short pidx;			/* plist block allocation */
+#endif
   struct SOAP_ENV__Header *header;
   struct SOAP_ENV__Fault *fault;
   int idnum;
@@ -1892,11 +1939,13 @@ struct SOAP_STD_API soap
   size_t (*fmimeread)(struct soap*, void*, char*, size_t);
   int (*fmimewrite)(struct soap*, void*, const char*, size_t);
 #endif
-  SOAP_SOCKET master;
-  SOAP_SOCKET socket;
+  SOAP_SOCKET master;	/* socket bound to TCP/IP port */
+  SOAP_SOCKET socket;	/* socket to send and receive */
+  SOAP_SOCKET sendsk;	/* socket to send (overrides ::socket) */
+  SOAP_SOCKET recvsk;	/* socket to receive (overrides ::socket) */
 #if defined(__cplusplus) && !defined(WITH_LEAN) && !defined(WITH_COMPAT)
-  std::ostream *os;
-  std::istream *is;
+  std::ostream *os;	/* stream to send */
+  std::istream *is;	/* stream to receive */
 #else
   void *os;		/* preserve struct size */
   void *is;		/* preserve struct size */
@@ -1905,14 +1954,14 @@ struct SOAP_STD_API soap
   int sendfd;		/* WinCE FD to send */
   int recvfd;		/* WinCE FD to receive */
 #else
-  FILE *sendfd;
-  FILE *recvfd;
+  FILE *sendfd;		/* FD to send */
+  FILE *recvfd;		/* FD to receive */
 #endif
   size_t bufidx;	/* index in soap.buf[] */
   size_t buflen;	/* length of soap.buf[] content */
   soap_wchar ahead;	/* parser lookahead */
   short cdata;		/* CDATA parser state */
-  short body;		/* parsed XML element has a body or not */
+  short body;		/* HTTP or XML element has a body (1) or not (0) */
   unsigned int level;	/* XML nesting level */
   size_t count;		/* message length counter */
   size_t length;	/* message length as set by HTTP header */
@@ -1988,7 +2037,7 @@ struct SOAP_STD_API soap
   int cookie_max;
 #endif
 #ifndef WITH_NOIO
-  int ipv6_multicast_if; /* in6addr->sin6_scope_id IPv6 value */
+  unsigned int ipv6_multicast_if; /* in_addr_t in6addr->sin6_scope_id IPv6 value */
   char* ipv4_multicast_if; /* IP_MULTICAST_IF IPv4 setsockopt interface_addr */
   unsigned char ipv4_multicast_ttl; /* IP_MULTICAST_TTL value 0..255 */
 #ifdef WITH_IPV6
@@ -2122,13 +2171,17 @@ soap_wchar soap_get0(struct soap*);
 soap_wchar soap_get1(struct soap*);
 #endif
 
+#define soap_versioning_paste(name, ext) name##_LIBRARY_VERSION_REQUIRED_##ext
+#define soap_versioning_ext(name, ext) soap_versioning_paste(name, ext)
+#define soap_versioning(name) soap_versioning_ext(name, GSOAP_VERSION)
+
 #define soap_init(soap) soap_init1(soap, SOAP_IO_DEFAULT)
 #define soap_init1(soap, mode) soap_init2(soap, mode, mode)
-#define soap_init2(soap, imode, omode) soap_init0(soap, GSOAP_H_VERSION, imode, omode)
+#define soap_init2(soap, imode, omode) soap_versioning(soap_init)(soap, imode, omode)
 
 #define soap_new() soap_new1(SOAP_IO_DEFAULT)
 #define soap_new1(mode) soap_new2(mode, mode)
-#define soap_new2(imode, omode) soap_new0(GSOAP_H_VERSION, imode, omode)
+#define soap_new2(imode, omode) soap_versioning(soap_new)(imode, omode)
 
 #define soap_revget1(soap) ((soap)->bufidx--)
 #define soap_unget(soap, c) ((soap)->ahead = c)
@@ -2162,6 +2215,18 @@ soap_wchar soap_get1(struct soap*);
  SOAP_FMAC1 unsigned long SOAP_FMAC2 soap_strtoul(const char *s, char **t, int b);
 #endif
 
+#ifdef WIN32
+# define soap_strtoll _strtoi64
+#else
+# define soap_strtoll strtoll
+#endif
+
+#ifdef WIN32
+# define soap_strtoull _strtoui64
+#else
+# define soap_strtoull strtoull
+#endif
+
 #if defined(WITH_OPENSSL)
 # define soap_random soap_rand()
 SOAP_FMAC1 int SOAP_FMAC2 soap_rand(void);
@@ -2192,20 +2257,26 @@ SOAP_FMAC1 int SOAP_FMAC2 soap_rand(void);
 /* soap_traverse() traversal/walker routines take walker function arguments */
 typedef void soap_walker(struct soap*, void*, int, const char*, const char*);
 
-SOAP_FMAC1 void SOAP_FMAC2 soap_header(struct soap*);
-SOAP_FMAC1 void SOAP_FMAC2 soap_fault(struct soap*);
-SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultcode(struct soap*);
-SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultsubcode(struct soap*);
-SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultstring(struct soap*);
-SOAP_FMAC1 const char** SOAP_FMAC2 soap_faultdetail(struct soap*);
-SOAP_FMAC1 const char* SOAP_FMAC2 soap_check_faultsubcode(struct soap*);
-SOAP_FMAC1 const char* SOAP_FMAC2 soap_check_faultdetail(struct soap*);
+SOAP_FMAC5 int SOAP_FMAC6 soap_serve(struct soap *soap);
+SOAP_FMAC5 int SOAP_FMAC6 soap_serve_request(struct soap *soap);
+
+#ifndef WITH_NOGLOBAL
+SOAP_FMAC3 void SOAP_FMAC4 soap_header(struct soap*);
+SOAP_FMAC3 void SOAP_FMAC4 soap_fault(struct soap*);
+SOAP_FMAC3 const char** SOAP_FMAC4 soap_faultcode(struct soap*);
+SOAP_FMAC3 const char** SOAP_FMAC4 soap_faultsubcode(struct soap*);
+SOAP_FMAC3 const char** SOAP_FMAC4 soap_faultstring(struct soap*);
+SOAP_FMAC3 const char** SOAP_FMAC4 soap_faultdetail(struct soap*);
+SOAP_FMAC3 const char* SOAP_FMAC4 soap_check_faultsubcode(struct soap*);
+SOAP_FMAC3 const char* SOAP_FMAC4 soap_check_faultdetail(struct soap*);
+SOAP_FMAC3 void SOAP_FMAC4 soap_serializefault(struct soap*);
+#endif
+
 SOAP_FMAC1 void SOAP_FMAC2 soap_serializeheader(struct soap*);
-SOAP_FMAC1 int SOAP_FMAC2 soap_putheader(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_getheader(struct soap*);
-SOAP_FMAC1 void SOAP_FMAC2 soap_serializefault(struct soap*);
-SOAP_FMAC1 int SOAP_FMAC2 soap_putfault(struct soap*);
+SOAP_FMAC1 int SOAP_FMAC2 soap_putheader(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_getfault(struct soap*);
+SOAP_FMAC1 int SOAP_FMAC2 soap_putfault(struct soap*);
 
 SOAP_FMAC1 void SOAP_FMAC2 soap_ssl_init(void);
 SOAP_FMAC1 int SOAP_FMAC2 soap_poll(struct soap*);
@@ -2282,6 +2353,7 @@ SOAP_FMAC1 int SOAP_FMAC2 soap_begin_count(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_end_count(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_begin_send(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_end_send(struct soap*);
+SOAP_FMAC1 int SOAP_FMAC2 soap_end_send_flush(struct soap*);
 
 SOAP_FMAC1 const struct soap_code_map* SOAP_FMAC2 soap_code(const struct soap_code_map*, const char*);
 SOAP_FMAC1 long SOAP_FMAC2 soap_code_int(const struct soap_code_map*, const char*, long);
@@ -2327,13 +2399,13 @@ SOAP_FMAC1 char* SOAP_FMAC2 soap_putoffsets(struct soap*, const int *, int);
 SOAP_FMAC1 int SOAP_FMAC2 soap_closesock(struct soap*);
 SOAP_FMAC1 int SOAP_FMAC2 soap_force_closesock(struct soap*);
 
-SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_new0(int, soap_mode, soap_mode);
+SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_versioning(soap_new)(soap_mode, soap_mode);
 SOAP_FMAC1 void SOAP_FMAC2 soap_free(struct soap*);
 SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_copy(const struct soap*);
 SOAP_FMAC1 struct soap *SOAP_FMAC2 soap_copy_context(struct soap*, const struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_copy_stream(struct soap*, struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_free_stream(struct soap*);
-SOAP_FMAC1 int SOAP_FMAC2 soap_init0(struct soap*, int, soap_mode, soap_mode);
+SOAP_FMAC1 void SOAP_FMAC2 soap_versioning(soap_init)(struct soap*, soap_mode, soap_mode);
 SOAP_FMAC1 void SOAP_FMAC2 soap_done(struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_cleanup(struct soap*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_begin(struct soap*);
@@ -2392,6 +2464,7 @@ SOAP_FMAC1 wchar_t* SOAP_FMAC2 soap_wstring_in(struct soap*, int, long, long);
 
 SOAP_FMAC1 int SOAP_FMAC2 soap_match_namespace(struct soap*, const char *, const char*, size_t n1, size_t n2);
 
+SOAP_FMAC1 void SOAP_FMAC2 soap_set_version(struct soap*, short);
 SOAP_FMAC1 int SOAP_FMAC2 soap_set_namespaces(struct soap*, const struct Namespace*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_set_local_namespaces(struct soap*);
 
@@ -2416,7 +2489,7 @@ SOAP_FMAC1 void SOAP_FMAC2 soap_end_block(struct soap*, struct soap_blist*);
 SOAP_FMAC1 void SOAP_FMAC2 soap_update_pointers(struct soap *soap, char *start, char *end, char *p1, char *p2);
 
 SOAP_FMAC1 int SOAP_FMAC2 soap_envelope_begin_out(struct soap*);
-SOAP_FMAC1 int soap_envelope_end_out(struct soap*);
+SOAP_FMAC1 int SOAP_FMAC2 soap_envelope_end_out(struct soap*);
 
 SOAP_FMAC1 char * SOAP_FMAC2 soap_get_http_body(struct soap*);
 
