@@ -408,7 +408,8 @@ if (!soap_valid_socket(soap_bind(soap, NULL, port, 100)))
 To implement a separate server for handling relayed SOAP response messages
 based on the ReplyTo information header in the request message, the gSOAP
 header file should include a one-way service operation for the response
-message.
+message. These one-way response service operations are automatically generated
+with wsdl2h option -b.
 
 For example, suppose a service operation returns an exampleResponse message. We
 declare the one-way exampleResponse operation as follows:
@@ -456,8 +457,8 @@ service operation. This allows us both to implement a one-way service operation
 that accepts faults and to automatically generate the fault struct for fault
 data storage and manipulation. 
 
-The fault operation in the header file should be declared as follows (for the
-2004/08 standard):
+The fault operation in the WS-Addressing files (wsa5.h etc.) is declared as
+follows (here shown for the 2004/08 standard):
 
 @code
 //gsoap SOAP_ENV service method-action: Fault http://schemas.xmlsoap.org/ws/2004/08/addressing/fault
@@ -478,10 +479,11 @@ int SOAP_ENV__Fault
 Because each service operation has a struct to hold its input parameters, we
 automatically generate the (original) SOAP_ENV__Fault struct on the fly!
 
-Note: it is important to associate the wsa fault action with this operation as
-shown above.
+It is important to associate the wsa fault action with this operation as shown
+above.
 
-The implementation of the service operation in the server code is:
+The implementation of the Fault service operation in your server code should be
+something like:
 
 @code
 int SOAP_ENV__Fault(struct soap *soap, char *faultcode, char *faultstring, char *faultactor, struct SOAP_ENV__Detail *detail, struct SOAP_ENV__Code *SOAP_ENV__Code, struct SOAP_ENV__Reason *SOAP_ENV__Reason, char *SOAP_ENV__Node, char *SOAP_ENV__Role, struct SOAP_ENV__Detail *SOAP_ENV__Detail)
@@ -539,6 +541,9 @@ const char *soap_wsa_noneURI = "addressing/none not supported";
 const char *soap_wsa_faultAction = "http://schemas.xmlsoap.org/ws/2004/08/addressing/fault";
 #endif
 
+/** anonymous URI of 2004 and 2005 schemas */
+const char *soap_wsa_allAnonymousURI = "http://schemas.xmlsoap.org/ws/2004/03/addressing/role/anonymous http://schemas.xmlsoap.org/ws/2004/08/addressing/role/anonymous http://www.w3.org/2005/08/addressing/anonymous";
+
 /******************************************************************************\
  *
  *	Static protos
@@ -565,11 +570,11 @@ static int soap_wsa_alloc_header(struct soap *soap);
 @brief Generates a random UUID (UUID algorithm version 4). Compile all source
 codes with -DWITH_OPENSSL for better randomness results.
 @param soap context
-@return UUID "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx"
+@return UUID "urn:uuid:xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx"
 */
 const char*
 soap_wsa_rand_uuid(struct soap *soap)
-{ char *uuid = (char*)soap_malloc(soap, 37);
+{ char *uuid = (char*)soap_malloc(soap, 48);
   int r1, r2, r3, r4;
 #ifdef WITH_OPENSSL
   r1 = soap_random;
@@ -594,7 +599,7 @@ soap_wsa_rand_uuid(struct soap *soap)
 #endif
   r3 = soap_random;
   r4 = soap_random;
-  sprintf(uuid, "%8.8x-%4.4hx-4%3.3hx-%4.4hx-%4.4hx%8.8x", r1, (short)(r2 >> 16), (short)r2 >> 4, ((short)(r3 >> 16) & 0x3FFF) | 0x8000, (short)r3, r4);
+  sprintf(uuid, "urn:uuid:%8.8x-%4.4hx-4%3.3hx-%4.4hx-%4.4hx%8.8x", r1, (short)(r2 >> 16), ((short)r2 >> 4) & 0x0FFF, ((short)(r3 >> 16) & 0x3FFF) | 0x8000, (short)r3, r4);
   DBGFUN1("soap_wsa_rand_uuid", "%s", uuid);
   return uuid;
 }
@@ -624,14 +629,15 @@ soap_wsa_request(struct soap *soap, const char *id, const char *to, const char *
   if (soap_wsa_alloc_header(soap))
     return soap->error;
   soap->header->SOAP_WSA(MessageID) = soap_strdup(soap, id);
-  if (!to)
-    to = (char*)soap_wsa_anonymousURI;
-  soap->header->SOAP_WSA(To) = soap_strdup(soap, to);
+  if (to)
+    soap->header->SOAP_WSA(To) = soap_strdup(soap, to);
+  else /* this is optional */
+    soap->header->SOAP_WSA(To) = (char*)soap_wsa_anonymousURI;
   soap->header->SOAP_WSA(Action) = soap_strdup(soap, action);
   soap->header->SOAP_WSA(RelatesTo) = NULL;
   soap->header->SOAP_WSA(From) = NULL;
-  soap->header->SOAP_WSA(ReplyTo) = NULL;
   soap->header->SOAP_WSA(FaultTo) = NULL;
+  soap_wsa_add_ReplyTo(soap, NULL);
   return soap_wsa_check(soap);
 }
 
@@ -673,7 +679,7 @@ soap_wsa_add_NoReply(struct soap *soap)
 @fn int soap_wsa_add_ReplyTo(struct soap *soap, const char *replyTo)
 @brief Sets WS-Addressing ReplyTo header for request message.
 @param soap context
-@param[in] replyTo endpoint URI 
+@param[in] replyTo endpoint URI or NULL for anonymous
 @return SOAP_OK or SOAP_ERR
 
 Use soap_wsa_request to populate the WS-Addressing header.
@@ -682,11 +688,17 @@ int
 soap_wsa_add_ReplyTo(struct soap *soap, const char *replyTo)
 { if (!soap->header)
     return SOAP_ERR;
+#ifndef SOAP_WSA_2005
+  if (!replyTo)
+    replyTo = soap_wsa_anonymousURI;
+#endif
   if (replyTo)
   { soap->header->SOAP_WSA(ReplyTo) = (SOAP_WSA_(,ReplyTo)*)soap_malloc(soap, sizeof(SOAP_WSA_(,ReplyTo)));
     SOAP_WSA_(soap_default,EndpointReferenceType)(soap, soap->header->SOAP_WSA(ReplyTo));
     soap->header->SOAP_WSA(ReplyTo)->Address = soap_strdup(soap, replyTo);
   }
+  else
+    soap->header->SOAP_WSA(ReplyTo) = NULL;
   return SOAP_OK;
 }
 
@@ -694,7 +706,7 @@ soap_wsa_add_ReplyTo(struct soap *soap, const char *replyTo)
 @fn int soap_wsa_add_FaultTo(struct soap *soap, const char *faultTo)
 @brief Sets WS-Addressing FaultTo header for request message.
 @param soap context
-@param[in] faultTo endpoint URI 
+@param[in] faultTo endpoint URI or NULL for remove faultTo
 @return SOAP_OK or SOAP_ERR
 
 Use soap_wsa_request to populate the WS-Addressing header first.
@@ -708,6 +720,8 @@ soap_wsa_add_FaultTo(struct soap *soap, const char *faultTo)
     SOAP_WSA_(soap_default,EndpointReferenceType)(soap, soap->header->SOAP_WSA(FaultTo));
     soap->header->SOAP_WSA(FaultTo)->Address = soap_strdup(soap, faultTo);
   }
+  else
+    soap->header->SOAP_WSA(FaultTo) = NULL;
   return SOAP_OK;
 }
 
@@ -732,6 +746,58 @@ soap_wsa_add_RelatesTo(struct soap *soap, const char *relatesTo)
   return SOAP_OK;
 }
 
+/**
+@fn const char *soap_wsa_From(struct soap *soap)
+@brief Returns WS-Addressing From header.
+@param soap context
+@return From string or NULL
+*/
+const char*
+soap_wsa_From(struct soap *soap)
+{ if (!soap->header || !soap->header->SOAP_WSA(From))
+    return NULL;
+  return soap->header->SOAP_WSA(From)->Address;
+}
+
+/**
+@fn const char *soap_wsa_ReplyTo(struct soap *soap)
+@brief Returns WS-Addressing ReplyTo header.
+@param soap context
+@return From string or NULL
+*/
+const char*
+soap_wsa_ReplyTo(struct soap *soap)
+{ if (!soap->header || !soap->header->SOAP_WSA(ReplyTo))
+    return NULL;
+  return soap->header->SOAP_WSA(ReplyTo)->Address;
+}
+
+/**
+@fn const char *soap_wsa_FaultTo(struct soap *soap)
+@brief Returns WS-Addressing FaultTo header.
+@param soap context
+@return From string or NULL
+*/
+const char*
+soap_wsa_FaultTo(struct soap *soap)
+{ if (!soap->header || !soap->header->SOAP_WSA(FaultTo))
+    return NULL;
+  return soap->header->SOAP_WSA(FaultTo)->Address;
+}
+
+/**
+@fn const char *soap_wsa_RelatesTo(struct soap *soap)
+@brief Returns WS-Addressing RelatesTo header.
+@param soap context
+@return From string or NULL
+*/
+const char*
+soap_wsa_RelatesTo(struct soap *soap)
+{ if (!soap->header || !soap->header->SOAP_WSA(RelatesTo))
+    return NULL;
+  return soap->header->SOAP_WSA(RelatesTo)->__item;
+}
+
 /******************************************************************************\
  *
  *	Server-side Check and Reply
@@ -800,20 +866,48 @@ soap_wsa_reply(struct soap *soap, const char *id, const char *action)
     SOAP_WSA_(soap_default_,RelatesTo)(soap, newheader->SOAP_WSA(RelatesTo));
     newheader->SOAP_WSA(RelatesTo)->__item = oldheader->SOAP_WSA(MessageID);
   }
-  if (oldheader && oldheader->SOAP_WSA(ReplyTo) && oldheader->SOAP_WSA(ReplyTo)->Address && strcmp(oldheader->SOAP_WSA(ReplyTo)->Address, soap_wsa_anonymousURI))
+#ifdef SOAP_WSA_2005 
+  /* WCF Interoperability:
+     ChannelInstance is required when the WCF Application hosts multiple
+     Callback Channels within the same application. The
+     ReferenceParameters->ChannelInstance element serves as a hint to the WCF
+     Client dispatcher, as to which WCF callback instance a received SOAP
+     Envelope belongs to ChannelInstance is declared as a pointer, so it is
+     essentially an optional element. Tests with Applications not requiring
+     ChannelInstance have also been done for the following fix.
+  */
+  if (oldheader && oldheader->SOAP_WSA(ReplyTo) && oldheader->SOAP_WSA(ReplyTo)->ReferenceParameters && oldheader->SOAP_WSA(ReplyTo)->ReferenceParameters->chan__ChannelInstance)
+  { if (newheader)
+    { if (!newheader->chan__ChannelInstance)
+      { newheader->chan__ChannelInstance = (struct chan__ChannelInstanceType*)soap_malloc(soap, sizeof(struct chan__ChannelInstanceType));
+        if (newheader->chan__ChannelInstance)
+	{ soap_default_chan__ChannelInstanceType(soap, newheader->chan__ChannelInstance);
+          newheader->chan__ChannelInstance->__item = *(oldheader->SOAP_WSA(ReplyTo)->ReferenceParameters->chan__ChannelInstance);
+          newheader->chan__ChannelInstance->wsa5__IsReferenceParameter = _wsa5__IsReferenceParameter__true;
+        }
+      }
+      else
+      { newheader->chan__ChannelInstance->__item = *(oldheader->SOAP_WSA(ReplyTo)->ReferenceParameters->chan__ChannelInstance);
+        newheader->chan__ChannelInstance->wsa5__IsReferenceParameter = _wsa5__IsReferenceParameter__true;
+      }
+    }
+  }
+#endif
+  if (oldheader && oldheader->SOAP_WSA(ReplyTo) && oldheader->SOAP_WSA(ReplyTo)->Address && !soap_tagsearch(soap_wsa_allAnonymousURI, oldheader->SOAP_WSA(ReplyTo)->Address))
   { newheader->SOAP_WSA(To) = oldheader->SOAP_WSA(ReplyTo)->Address;
     /* (re)connect to ReplyTo endpoint if From != ReplyTo */
     if (!oldheader->SOAP_WSA(From) || !oldheader->SOAP_WSA(From)->Address || strcmp(oldheader->SOAP_WSA(From)->Address, oldheader->SOAP_WSA(ReplyTo)->Address))
     { struct soap *reply_soap = soap_copy(soap);
       if (reply_soap)
       { soap_copy_stream(reply_soap, soap);
-        soap_clr_omode(reply_soap, SOAP_ENC_MIME | SOAP_ENC_DIME | SOAP_ENC_MTOM);
 	soap_free_stream(soap); /* prevents close in soap_connect() below */
-        newheader->SOAP_WSA(FaultTo) = oldheader->SOAP_WSA(FaultTo);
-        soap->header = newheader;
         if (soap_connect(soap, newheader->SOAP_WSA(To), newheader->SOAP_WSA(Action)))
-        { int err; 
-          soap_copy_stream(soap, reply_soap);
+        { int err;
+	  soap_copy_stream(soap, reply_soap);
+	  soap_free_stream(reply_soap);
+          soap_end(reply_soap);
+          soap_free(reply_soap);
+	  soap->header = oldheader;
 #if defined(SOAP_WSA_2005)
           err = soap_wsa_error(soap, SOAP_WSA(DestinationUnreachable), newheader->SOAP_WSA(To));
 #elif defined(SOAP_WSA_2003)
@@ -821,17 +915,12 @@ soap_wsa_reply(struct soap *soap, const char *id, const char *action)
 #else
           err = soap_wsa_error(soap, SOAP_WSA(DestinationUnreachable));
 #endif
-	  soap_free_stream(reply_soap);
-          soap_end(reply_soap);
-          soap_free(reply_soap);
           soap->header = NULL;
           return err;
         }
         if (soap_valid_socket(reply_soap->socket))
-        { soap_send_empty_response(reply_soap, SOAP_OK);	/* HTTP ACCEPTED */
-          soap_closesock(reply_soap);
-        }
-	soap_free_stream(reply_soap);
+          soap_send_empty_response(reply_soap, SOAP_OK);	/* HTTP ACCEPTED */
+        soap->header = newheader;
         soap_end(reply_soap);
         soap_free(reply_soap);
         data->fresponse = soap->fresponse;
@@ -888,6 +977,17 @@ soap_wsa_fault_subcode_action(struct soap *soap, int flag, const char *faultsubc
   if (!data)
     return soap->error = SOAP_PLUGIN_ERROR;
   oldheader = soap->header;
+  /* no FaultTo: use ReplyTo */
+  if (oldheader && oldheader->SOAP_WSA(ReplyTo) && (!oldheader->SOAP_WSA(FaultTo) || soap_tagsearch(soap_wsa_allAnonymousURI, oldheader->SOAP_WSA(FaultTo)->Address)))
+  { if (!oldheader->SOAP_WSA(FaultTo))
+    { oldheader->SOAP_WSA(FaultTo) = (SOAP_WSA_(,FaultTo)*)soap_malloc(soap, sizeof(SOAP_WSA_(,FaultTo)));
+      SOAP_WSA_(soap_default,EndpointReferenceType)(soap, soap->header->SOAP_WSA(FaultTo));
+    }
+    oldheader->SOAP_WSA(FaultTo)->Address = oldheader->SOAP_WSA(ReplyTo)->Address;
+  }
+  if (oldheader && oldheader->SOAP_WSA(FaultTo))
+  { DBGLOG(TEST, SOAP_MESSAGE(fdebug, "WSA FaultTo='%s'\n", oldheader->SOAP_WSA(FaultTo)->Address));
+  }
   if (oldheader && oldheader->SOAP_WSA(FaultTo) && !strcmp(oldheader->SOAP_WSA(FaultTo)->Address, soap_wsa_noneURI))
     return soap_send_empty_response(soap, SOAP_OK);	/* HTTP ACCEPTED */
   soap->header = NULL;
@@ -904,7 +1004,7 @@ soap_wsa_fault_subcode_action(struct soap *soap, int flag, const char *faultsubc
   }
   /* header->wsa__MessageID = "..."; */
   newheader->SOAP_WSA(Action) = (char*)soap_wsa_faultAction;
-  if (oldheader && oldheader->SOAP_WSA(FaultTo) && oldheader->SOAP_WSA(FaultTo)->Address && strcmp(oldheader->SOAP_WSA(FaultTo)->Address, soap_wsa_anonymousURI))
+  if (oldheader && oldheader->SOAP_WSA(FaultTo) && oldheader->SOAP_WSA(FaultTo)->Address && !soap_tagsearch(soap_wsa_allAnonymousURI, oldheader->SOAP_WSA(FaultTo)->Address))
   { newheader->SOAP_WSA(To) = oldheader->SOAP_WSA(FaultTo)->Address;
     /* (re)connect to FaultTo endpoint if From != FaultTo */
     if (!oldheader->SOAP_WSA(From) || !oldheader->SOAP_WSA(From)->Address || strcmp(oldheader->SOAP_WSA(From)->Address, oldheader->SOAP_WSA(FaultTo)->Address))
@@ -1033,7 +1133,7 @@ soap_wsa_receiver_fault(struct soap *soap, const char *faultstring, const char *
 int
 soap_wsa_check_fault(struct soap *soap, SOAP_WSA(FaultCodesType) *fault, const char **info)
 { if (soap->error && soap->fault && soap->fault->SOAP_ENV__Code)
-  { const char *code = *soap_faultsubcode(soap);
+  { const char *code = soap_check_faultsubcode(soap);
     if (code)
     { SOAP_WSA__(soap_s2,FaultCodesType)(soap, code, fault);
       if (info)
@@ -1091,7 +1191,7 @@ soap_wsa_check_fault(struct soap *soap, char **fault)
 int
 soap_wsa_check_fault(struct soap *soap, SOAP_WSA(FaultSubcodeValues) *fault)
 { if (soap->error && soap->fault && soap->fault->SOAP_ENV__Code)
-  { const char *code = *soap_faultsubcode(soap);
+  { const char *code = soap_check_faultsubcode(soap);
     if (code)
     { SOAP_WSA__(soap_s2,FaultSubcodeValues)(soap, code, fault);
       return soap->error;
@@ -1116,7 +1216,6 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
   /* populate the SOAP Fault as per WS-Addressing spec */
   switch (fault)
   { case SOAP_WSA(InvalidAddressingHeader):
-      soap_wsa_sender_fault_subcode(soap, code, "A header representing a Message Addressing Property is not valid and the message cannot be processed.", NULL);
       soap_faultdetail(soap);
       if (soap->version == 1)
       { soap->fault->detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemHeaderQName);
@@ -1126,6 +1225,7 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
       { soap->fault->SOAP_ENV__Detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemHeaderQName);
         soap->fault->SOAP_ENV__Detail->fault = (void*)info;
       }
+      soap_wsa_sender_fault_subcode(soap, code, "A header representing a Message Addressing Property is not valid and the message cannot be processed.", NULL);
       break;
     case SOAP_WSA(InvalidAddress):
       soap_wsa_sender_fault_subcode(soap, code, "Invalid address.", NULL);
@@ -1146,7 +1246,6 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
       soap_wsa_sender_fault_subcode(soap, code, "Action and SOAP action of the message do not match.", NULL);
       break;
     case SOAP_WSA(MessageAddressingHeaderRequired):
-      soap_wsa_sender_fault_subcode(soap, code, "A required header representing a Message Addressing Property is not present.", NULL);
       soap_faultdetail(soap);
       if (soap->version == 1)
       { soap->fault->detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemHeaderQName);
@@ -1156,9 +1255,9 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
       { soap->fault->SOAP_ENV__Detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemHeaderQName);
         soap->fault->SOAP_ENV__Detail->fault = (void*)info;
       }
+      soap_wsa_sender_fault_subcode(soap, code, "A required header representing a Message Addressing Property is not present.", NULL);
       break;
     case SOAP_WSA(DestinationUnreachable):
-      soap_wsa_sender_fault_subcode(soap, code, "No route can be determined to reach [destination]", NULL);
       soap_faultdetail(soap);
       if (soap->version == 1)
       { soap->fault->detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemIRI);
@@ -1168,9 +1267,9 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
       { soap->fault->SOAP_ENV__Detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemIRI);
         soap->fault->SOAP_ENV__Detail->fault = (void*)info;
       }
+      soap_wsa_sender_fault_subcode(soap, code, "No route can be determined to reach [destination]", NULL);
       break;
     case SOAP_WSA(ActionNotSupported):
-      soap_wsa_sender_fault_subcode(soap, code, "The [action] cannot be processed at the receiver.", NULL);
       soap_faultdetail(soap);
       if (soap->version == 1)
       { soap->fault->detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemAction);
@@ -1184,9 +1283,9 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
         SOAP_WSA_(soap_default_,ProblemAction)(soap, (SOAP_WSA_(,ProblemAction)*)soap->fault->SOAP_ENV__Detail->fault);
         ((SOAP_WSA_(,ProblemAction)*)soap->fault->SOAP_ENV__Detail->fault)->Action = (char*)info;
       }
+      soap_wsa_sender_fault_subcode(soap, code, "The [action] cannot be processed at the receiver.", NULL);
       break;
     case SOAP_WSA(EndpointUnavailable):
-      soap_wsa_receiver_fault_subcode(soap, code, "The endpoint is unable to process the message at this time.", NULL);
       soap_faultdetail(soap);
       if (soap->version == 1)
       { soap->fault->detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemIRI);
@@ -1196,6 +1295,7 @@ soap_wsa_error(struct soap *soap, SOAP_WSA(FaultCodesType) fault, const char *in
       { soap->fault->SOAP_ENV__Detail->__type = SOAP_WSA_(SOAP_TYPE_,ProblemIRI);
         soap->fault->SOAP_ENV__Detail->fault = (void*)info;
       }
+      soap_wsa_receiver_fault_subcode(soap, code, "The endpoint is unable to process the message at this time.", NULL);
       break;
     default:
       break;
